/**************************************************************************
 *  Nombre:  ibutton.c
 *  Version: 1.00
 *  Fecha:   30/5/07	
 *  Autor:   Ulises Bigliati
 *  Lenguaje:Ansi C
 *
 *  Descripcion
 * =======================================================================
 * El mdulo contine la definicion de las funciones y constantes necesarias 
 * para leer el iD de los iButtons de Dallas Semiconductors. 
 * ========================================================================
 * Revisiones:
 * 16/08/07:
 * 		Modificacin de tiempos de lectura principalmente en ibBitRead()
 * 		Modificacin de estructura en ibReset()
***************************************************************************/
#include "ibutton.h"
#include "gpio.h"
#include <string.h>

void usDelay(unsigned int);
unsigned char ibByteRead(void);
void ibByteWrite(unsigned char);
void ibBitWrite(  unsigned int);
unsigned int ibBitRead(void);
unsigned char crccheck (char * );

//--------------------------------------------------------------------------
// Genera el ciclo de reset sobre el bus 1-Wire y retorna:
// IB_READY(1):  presense pulse detectado
// IB_NOT_READY (0): no se detecta el presense pulse 

unsigned int ibReset(void){
   unsigned int result,i;
   
   NAsetGPIOpin(IB_OUT, 1); // bus low.
   usDelay(450); 			//  ~= 488 us
   
   NAsetGPIOpin(IB_OUT, 0); // bus high.
   usDelay(60); 			//  ~= 70 us

   		for(i=0;i<200;i++){	
		   NAgetGPIOpin(IB_IN, &result);// get presence pulse.
		   if (!result) 
				goto tpdl_ok;
   		}
		return IB_NOT_READY;

	tpdl_ok:  
   		for(i=0;i<200;i++){	
   			NAgetGPIOpin(IB_IN, &result);// Busco el recovery pulse
			if (result){
				usDelay(450);			 
				return  IB_READY;
			}
   		}
		return IB_NOT_READY;
}

/*Lee los 8 bytes del touch y deja la lectura en un vector local a la funcion
 *Llama a crccheck para validar la lectura, si CRC falla, la descarta  
 *Retorno: TRUE si CRC ok, False si CRC fallo 
 * */

unsigned int ibRead(unsigned char * ns){
		unsigned int result,i;
		unsigned char ibSerial[8];
					//Lectura
					ibByteWrite(0x33);
	   				for(i=0;i<8;i++){
				   		ibSerial[i]=ibByteRead();
	   				}
				   	//Calculo CRC 
					if (crccheck(ibSerial)==ibSerial[7]){
		   			//Impresion si CRC OK
		   				memcpy(ns, ibSerial, 8);
						result= IB_LECTURA_OK;	   	
					}
				   	else{	 	
						result= IB_LECTURA_ERROR;	//  	
				   	}
	return result;
}
//--------------------------------------------------------------------------
// Retorna:  un char con 8 bits leidos 
//--------------------------------------------------------------------------
unsigned char ibByteRead( void){
   uchar i;
   uchar result = 0;
	
   for (i = 0; i < 8; i++){
	    result |= ibBitRead() << i;  
   }
   return result;
}
/*********************************
; Devuelve un bit leido *
;*********************************/

 unsigned int ibBitRead(void){
   unsigned int result;

	NAsetGPIOpin(IB_OUT, 1);		//Inicio time slot, Bajo la linea.		
		//usDelay(2);					//***Leo sin retardos***
	NAsetGPIOpin(IB_OUT, 0);		//HI (Suelto la linea)
		//usDelay(15);					//***Leo sin retardos*** 
	NAgetGPIOpin(IB_IN, &result);	//Supongo tener un dato valido

	usDelay(30);					//Completo el time-slot     
	return result;					//Devuelve el bit leido  
}	
/*********************************
; Escribe un byte (normalente un comando) en el bus
; El LSB va primero
;*********************************/
void ibByteWrite( unsigned char command){
   uchar i;
	unsigned int bit=0;
		
   for (i = 0; i < 8; i++){
   		bit= (command >>i)& 1;
       ibBitWrite(bit) ;  
   }
}
//--------------------------------------------------------------------------
// Escribe un bit.
//--------------------------------------------------------------------------
 void ibBitWrite( unsigned int sendbit)
{
   // start timeslot.
   if (sendbit)
   {
    	NAsetGPIOpin(IB_OUT, 1) ; //Abajo
	    usDelay(1); //tw1L 	
       	NAsetGPIOpin(IB_OUT, 0) ; //Arriba
	    usDelay(50); //tw1H 
   }
   else
   {
	   NAsetGPIOpin(IB_OUT, 1);//Abajo
	   usDelay(60); //tw0L 
	   NAsetGPIOpin(IB_OUT, 0);//Arriba
   }

}




/******************************************
; Tabla para el Calculo de CRC de Dallas  *
;******************************************/

static uchar dscrc_table[] = {
        0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
      157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220,
       35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98,
      190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
       70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7,
      219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154,
      101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36,
      248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185,
      140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
       17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
      175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
       50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
      202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139,
       87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
      233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
      116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53};

/********************************************************************
* Rutina de clculo de CRC de Dallas Semiconductor para One Wire 
* Argumento: 
* 			puntero a caracter apuntando a la cadena que contiene la 
* 			lectura obtenida.
* Retorno: 
* 			CRC de 8 bits para los 7 bytes correspondientes a familia+id
*********************************************************************/
unsigned char crccheck(char *serie){
	unsigned char crc8=0;
	unsigned char i;

   	for(i=0;i<7;i++){
		crc8 = dscrc_table[crc8 ^ serie[i]];
	}
	return crc8;
}

void usDelay (unsigned int us){
	unsigned int i;
	for (i=0;i<us;i++); 
}
